/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/

#ifndef MAIN_H
#define MAIN_H

#include <QtGui/QMainWindow>
#include <thread_template.h>

class QTextEdit;
class QStackedWidget;

class StackedIconWidget;
class ResultsToSpectrum;
class SpectrumGui;
class GraphicalGuiServer;

#ifdef DATABASE
class ResultsTable;
#endif

namespace NSEvaluation {
	class EvaluationPath;
};

namespace NSScript {
	class ScriptEnvironment;
};

class XMLEditor : public QWidget {
	Q_OBJECT
	public:
		XMLEditor( QWidget* =0 );
		~XMLEditor();

	public slots:
		void setXML( QString );

	signals:
		void saveXML( QString );

	private slots:
		void commitChanges();

	private:
		QToolBar* _toolBar;
		QTextEdit* _textEdit;
};

class EodeMainWindow : public QMainWindow {
	Q_OBJECT
	public:
		EodeMainWindow( QWidget* =0 );
		~EodeMainWindow();

		StackedIconWidget* stacked;

		NSCore::Thread_Template<READER> *thread_reader;
		NSCore::Thread_Template<NSEvaluation::EvaluationPath> *thread_evaluation;
		NSCore::Thread_Template<ResultsToSpectrum> *thread_res2spec;
#ifdef DATABASE
		NSCore::Thread_Template<ResultsTable> *thread_table;
#endif
#ifdef WRITER
		NSCore::Thread_Template<WRITER> *thread_writer;
#endif

		QMap <QString,SpectrumGui*> _spectrumguis;

		NSScript::ScriptEnvironment* _engine;
	public slots:
		void testing();

		void openXMLFile();
		void openXMLFile( QString file );
		void saveXMLFile();

		void openXML( QString );

		void checkSpectra();
		void spectrumDeleted( QString );

	signals:
		void closed();

	private slots:
		//void setDocument( QString );
		void setApplicationTitle( QString ="" );

		void switchview( bool );

	protected:
		void closeEvent( QCloseEvent* );

	private:
		QStackedWidget* _stackedwidgets;
		StackedIconWidget* _iconwidget;
		XMLEditor* _textedit;
		GraphicalGuiServer* _guiserver;
};

#endif // MAIN_H
