/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/

#include <QtCore/QDebug>
#include <QtGui/QApplication>
#include <QtGui/QSplitter>
#include <QtCore/QLibrary>
#include <QtGui/QMainWindow>
#include <QtGui/QAction>
#include <QtGui/QMenuBar>
#include <QtGui/QMenu>
#include <QtGui/QIcon>
#include <QtGui/QFileDialog>
#include <QtXml/QDomDocument>
#include <QtCore/QFile>
#include <QtGui/QPlastiqueStyle> // Should be removed...
#include <QtGui/QScrollArea>
#include <QtGui/QTreeView>
#include <QtGui/QHeaderView>
#include <QtGui/QTextEdit>
#include <QtGui/QToolBar>
#include <QtGui/QMessageBox>
#include <QtGui/QLCDNumber>
#include <QtGui/QLabel>
#include <QtGui/QVBoxLayout>
#include <QtGui/QStackedWidget>

#include <evaluationpath.h>
#include <thread_template.h>
#include <osciwidget_v3.h>
#include <resulttospectrum.h>
#include <spectrummodel.h>
#include <spectrumgui.h>
#include <stackediconwidget.h>

#include <qtreeitemmodel.h>
#include <qtreeitemmodeldelegate.h>
#include <graphicalguiserver.h>

#include <scriptenvironment.h>
#include <scriptsnippet.h>
#include <scriptintrospection.h>
#include <scriptworkspace.h>


#include READER_INC
#ifdef READERGUI
#include READERGUI_INC
#endif

#ifdef WRITER
#include WRITER_INC
#include <diskio_gui.h>
#endif

#include "main.h"
#include "main.moc"

XMLEditor::XMLEditor( QWidget* p )
	: QWidget( p )
	, _toolBar( new QToolBar( this ) )
	, _textEdit( new QTextEdit( this ) )
{
	QVBoxLayout *_layout = new QVBoxLayout( this );
	_layout->addWidget( _toolBar, 1 );
	_layout->addWidget( _textEdit, 100 );

	_textEdit->setTabStopWidth( 4 );

	_toolBar->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );

	QAction* actioncommit = new QAction( "Commit changes", this );
	connect( actioncommit, SIGNAL( triggered() ), this, SLOT( commitChanges() ) );
	_toolBar->addAction( actioncommit );
}
XMLEditor::~XMLEditor() {
}

void XMLEditor::setXML( QString xml ) {
	qDebug() << "XMLEditor::setXML( QString xml )";
	//qDebug() << endl << xml << endl;
	QString html = xml;
	html.replace( '<', "&lt;" );
	html.replace( '>', "&gt;" );
	html.replace( '\n', "<br />\n" );
	html.replace( '\t', "&middot;\t" );
	html.replace( QRegExp( "(&lt;[^&]*&gt;)" ), "<font color='#0a0'>\\1</font> " );
	html.replace( QRegExp( "(\"[^\\\"]*\")" ), "<font color='#a00'>\\1</font> " );
	//qDebug() << endl << html << endl;
	_textEdit->setHtml( html );
}
void XMLEditor::commitChanges() {
	QString xml = _textEdit->toPlainText();
	qDebug() << endl << xml << endl;
	xml.replace( "·\t", "\t" );
	qDebug() << endl << xml << endl;
	// Isn't really working. But the ·'s get ignored eventually...
	emit saveXML( xml );
}

EodeMainWindow::EodeMainWindow( QWidget* p )
	: QMainWindow( p )
	, _stackedwidgets( new QStackedWidget( 0 ) )
	, _iconwidget( new StackedIconWidget( 0 ) )
	, _textedit( new XMLEditor( this ) )
	, _guiserver( new GraphicalGuiServer( this ) )
{
	qDebug() << "EodeMainWindow::EodeMainWindow(" << p << ") living in main-thread" << thread();

	//
	// Basic layout
	setCentralWidget( _stackedwidgets );

	_stackedwidgets->addWidget( _iconwidget );
	_stackedwidgets->addWidget( _textedit );

	setApplicationTitle();

	//
	// Configure QTextEdit
	connect( _textedit, SIGNAL( saveXML( QString ) ), this, SLOT( openXML( QString ) ) );

	//
	// Actions
	QAction* actionquit = new QAction( QIcon( ":/icons/kde/cr22-action-exit.png" ), "Quit", this );
		actionquit->setShortcut( Qt::CTRL|Qt::Key_Q );
		connect( actionquit, SIGNAL( triggered() ), this, SLOT( close() ) );
	QAction* actiontesting = new QAction( "Testing", this );
		connect( actiontesting, SIGNAL( triggered() ), this, SLOT( testing() ) );
	QAction* actionopenxml = new QAction( QIcon( ":/icons/kde/cr22-action-fileopen.png" ), "Open XML...", this );
		connect( actionopenxml, SIGNAL( triggered() ), this, SLOT( openXMLFile() ) );
	QAction* actionwritexml = new QAction( QIcon( ":/icons/kde/cr22-action-filesave.png" ), "Save Configuration to XML...", this );
		connect( actionwritexml, SIGNAL( triggered() ), this, SLOT( saveXMLFile() ) );

	QAction* actioncheckspectra = new QAction( QIcon( ":/icons/cr22-action-spectrum.png" ), "Check Spectra", this );
		connect( actioncheckspectra, SIGNAL( triggered() ), this, SLOT( checkSpectra() ) );

	QAction* actionswitchview = new QAction( "Show document as XML", this );
		actionswitchview->setCheckable( true );
		connect( actionswitchview, SIGNAL( toggled( bool ) ), this, SLOT( switchview( bool ) ) );

	//
	// Menus
	QMenu* menufile = menuBar()->addMenu( "&File" );
	menufile->addAction( actionopenxml );
	menufile->addAction( actionwritexml );
	//menufile->addSeparator();
	//menufile->addAction( actiontesting );
	menufile->addSeparator();
	menufile->addAction( actionquit );
	QMenu* menuedit = menuBar()->addMenu( "&Edit" );
	menuedit->addAction( actionswitchview );
	menuedit->addSeparator();
	menuedit->addAction( actioncheckspectra );

	//
	// Threads
	qDebug() << " * starting thread for the reader";
	thread_reader = new NSCore::Thread_Template<READER>( this );
	thread_reader->start();
	qDebug() << " * starting thread for the evaluation path";
	thread_evaluation = new NSCore::Thread_Template<NSEvaluation::EvaluationPath>( this );
	thread_evaluation->start();
	qDebug() << " * starting thread for the results2spectrum";
	thread_res2spec = new NSCore::Thread_Template<ResultsToSpectrum>( this );
	thread_res2spec->start();
#ifdef WRITER
	qDebug() << " * starting thread for the writer";
	thread_writer = new NSCore::Thread_Template<WRITER>( this );
	thread_writer->start();
#endif
	qDebug() << " * done, will wait for them to be started";
	while (
			( ! thread_reader->item() )
			||
			( ! thread_evaluation->item() )
			||
			( ! thread_res2spec->item() )
#ifdef WRITER
			|| ( ! thread_writer->item() )
#endif
			) {
		usleep( 20 );
	}
	qDebug() << " * Objects are there:"
		<< thread_reader->item()
		<< thread_evaluation->item()
		<< thread_res2spec->item()
#ifdef WRITER
		<< thread_writer->item()
#endif
		<< endl;

	if ( thread_reader->item()->objectName() == "" )
		thread_reader->item()->setObjectName( "reader" );
	thread_evaluation->item()->setObjectName( "eval" );
	thread_res2spec->item()->setObjectName( "res2spec" );

	///
	/// DataChain
	READER* reader = thread_reader->item();

#ifdef READERGUI
	READERGUI* readergui = new READERGUI( reader, 0 );
	addToolBar( readergui->toolbar() );
#endif

	QTreeItemModelDelegate* delegate = new QTreeItemModelDelegate( new GraphicalGuiServer( this ), 0 );

	/// Results to Spectrum
	ResultsToSpectrum* res2spec = thread_res2spec->item();
	connect( res2spec, SIGNAL( numberOfSpectraChanged() ), this, SLOT( checkSpectra() ) );
	connect( res2spec, SIGNAL( spectrumDeleted( QString ) ), this, SLOT( spectrumDeleted( QString ) ) );
	qDebug() << " res2spec =" << res2spec;

	OsciWidgetV3* osci = new OsciWidgetV3( 0 );

#ifdef WRITER
	NSDiskIO::DiskWriterGui* writergui = new NSDiskIO::DiskWriterGui( thread_writer->item(), 0 );
#endif

	///
	//Gui for all chain-members...
	NSCore::QTreeItemModel* chain_model = new NSCore::QTreeItemModel( this );
	chain_model->addToRootItem( thread_reader->item() );
	chain_model->addToRootItem( thread_evaluation->item() );
	chain_model->addToRootItem( thread_res2spec->item() );
#ifdef WRITER
	chain_model->addToRootItem( thread_writer->item() );
#endif

	QTreeView* chain_gui = new QTreeView( 0 );
	chain_gui->setModel( chain_model );
	chain_gui->setItemDelegate( delegate );
	chain_gui->setIconSize( QSize( 22,22 ) );
	chain_gui->resizeColumnToContents( 0 );

	///
	/// Gui of the chain
#ifdef READERGUI
	_iconwidget->addWidget( "Reader", readergui, ":/icons/cr48-action-load_pulses.png" );
#endif
	_iconwidget->addWidget( "Chain", chain_gui, ":/icons/kde/cr32-action-view_choose.png" );
	_iconwidget->addWidget( "Osci", osci, ":/icons/cr48-action-osci.png" );
#ifdef WRITER
	_iconwidget->addWidget( "Writer", writergui, "" );
#endif

	//
	// Plugging the chain together
	reader->registerConsumer( thread_evaluation->item() );

	thread_evaluation->item()->registerConsumer( res2spec );
#ifdef WRITER
	res2spec->registerConsumer( thread_writer->item() );
#endif
	thread_evaluation->item()->registerConsumer( osci->chainMember() );

	/// Scripting
	_engine = new NSScript::ScriptEnvironment( this );
	_engine->addQObject( _guiserver );
	_engine->addQObject( thread_reader->item() );
	_engine->addQObject( thread_evaluation->item() );
	_engine->addQObject( thread_res2spec->item() );

	QWidget* workspace = new NSScript::ScriptWorkSpace( _engine, 0 );
	connect( this, SIGNAL( closed() ), workspace, SLOT( close() ) );

	QAction* showscript = new QAction( "Toggle Script Workspace", this );
	showscript->setCheckable( true );
	showscript->setChecked( workspace->isVisible() );
	connect( showscript, SIGNAL( triggered( bool ) ), workspace, SLOT( setVisible( bool ) ) );
	connect( workspace, SIGNAL( visibilityChanged( bool ) ), showscript, SLOT( setChecked( bool ) ) );
	menuedit->addAction( showscript );

	QToolBar* toolbar = addToolBar( "Speeds" );
	QLCDNumber* lcd1 = new QLCDNumber();
	connect( reader, SIGNAL( countsPerSecond( double ) ), lcd1, SLOT( display( double ) ) );
	toolbar->addWidget( new QLabel( "Reader:" ) );
	toolbar->addWidget( lcd1 );
	QLCDNumber* lcd2 = new QLCDNumber();
	connect( thread_evaluation->item(), SIGNAL( countsPerSecond( double ) ), lcd2, SLOT( display( double ) ) );
	toolbar->addWidget( new QLabel( "Evaluation:" ) );
	toolbar->addWidget( lcd2 );
	QLCDNumber* lcd3 = new QLCDNumber();
	connect( res2spec, SIGNAL( countsPerSecond( double ) ), lcd3, SLOT( display( double ) ) );
	toolbar->addWidget( new QLabel( "Spectra:" ) );
	toolbar->addWidget( lcd3 );

	qDebug() << "EodeMainWindow::EodeMainWindow() finished...";
}
EodeMainWindow::~EodeMainWindow() {
}

void EodeMainWindow::testing() {
	qDebug() << endl << "EodeMainWindow::testing()";

	qDebug() << "EodeMainWindow::testing() finished." << endl;
}

void EodeMainWindow::openXMLFile() {
	QString path = QFileDialog::getOpenFileName( this, 0, 0, "XML-Files (*.xml)" );
	openXMLFile( path );
}

void EodeMainWindow::openXMLFile( QString path ) {
	qDebug() << "Will try to open" << path << "for reading";
	if ( QFile::exists( path ) ) {
		QFile file( path );
		if ( file.open( QIODevice::ReadOnly | QIODevice::Text ) ) {
			setApplicationTitle( path );
			QTextStream stream( &file );
			openXML( stream.readAll() );
			file.close();
		}
	}
}

void EodeMainWindow::openXML( QString xml ) {
	_textedit->setXML( xml );

	QDomDocument doc( "eode" );
	doc.setContent( xml );

	QDomNodeList list = doc.elementsByTagName( "reader" );
	for ( int i=0; i<list.size(); ++i )
		thread_reader->item()->fromXML( list.at( i ) );

	qDebug() << "Reading <evaluationpath />'s from XML";
	list = doc.elementsByTagName( "evaluationpath" );
	for ( int i=0; i<list.size(); ++i ) {
		qDebug() << i << list.at( i ).nodeName();
		thread_evaluation->item()->parseXML( list.at( i ) );
	}

	qDebug() << "Deleting all result2spectrum-paths";
	thread_res2spec->item()->removeAllPaths();
	while ( thread_res2spec->item()->spectra().size() > 0 )
		usleep( 100 );
	qDebug() << "Reading all <resultstospectrum />'s from XML";
	list = doc.elementsByTagName( "resultstospectrum" );
	for ( int i=0; i<list.size(); ++i ) {
		qDebug() << i << list.at( i ).nodeName();
		thread_res2spec->item()->addPath( list.at( i ) );
	}

	qDebug() << "Reading all scripts";
	list = doc.elementsByTagName( "scripts" );
	for ( int i=0; i<list.size(); ++i ) {
		qDebug() << i << list.at( i ).nodeName();
		_engine->fromXML( list.at( i ) );
	}
}

void EodeMainWindow::checkSpectra() {
	qDebug() << "EodeMainWindow::checkSpectra()";
	QStringList officialspectra = thread_res2spec->item()->spectra();
	qDebug() << officialspectra;
	foreach( QString item, _spectrumguis.keys() ) {
		qDebug() << "Checking wether" << item << " is a gui without a model";
		if ( ! officialspectra.contains( item ) ) {
			qDebug() << " Apparently it is :-(";
			delete _spectrumguis.take( item );
		}
	}
	foreach( QString item, officialspectra ) {
		//qDebug() << "Checking wether" << item << "has a gui.";
		if ( ! _spectrumguis.keys().contains( item ) ) {
			//qDebug() << " Apparently not, will create a gui";
			SpectrumGui* tmp = new SpectrumGui( 0 );
			tmp->setModel( thread_res2spec->item()->spectrum( item ) );
			_spectrumguis[ item ] = tmp;
			tmp->show();
			_iconwidget->addWidget( item, tmp, ":/icons/cr48-action-spectrum.png" );
		}
	}
}

void EodeMainWindow::spectrumDeleted( QString spec ) {
	qDebug() << "EodeMainWindow::spectrumDeleted(" << spec << ")";
	SpectrumGui* tmp = _spectrumguis.take( spec );
	qDebug() << " got" << tmp;
	delete tmp;
}

void EodeMainWindow::saveXMLFile() {
	QString path = QFileDialog::getSaveFileName( this, 0, 0, "XML-Files (*.xml)" );
	if ( path.isEmpty() )
		return;

	QFile file( path );
	if ( file.open( QIODevice::WriteOnly | QIODevice::Text ) ) {
		QTextStream stream( &file );
		stream << QString( "<eode>\n%1\n%2\n%3\n%4\n</eode>" ).arg(
				thread_reader->item()->toXML()
			).arg(
				thread_evaluation->item()->toXML()
			).arg(
				thread_res2spec->item()->toXML()
			 ).arg( 
				_engine->toXML()
			).replace( "><", ">\n<" );
		file.close();
	}
}

void EodeMainWindow::setApplicationTitle( QString n ) {
	if ( n == "" )
		n = "New untitled document";
	setWindowTitle( QString( "%1[*] - EPOS Offline Data Evaluation" ).arg( n ) );
	setWindowModified( true );
}

void EodeMainWindow::closeEvent( QCloseEvent* ) {
	emit closed();
}

void EodeMainWindow::switchview( bool n ) {
	_stackedwidgets->setCurrentIndex( n );
}


int main( int argc, char** argv ) {

	qDebug() << endl << "EvaluationChain starting...";

	QApplication app( argc, argv );

	QStringList args = app.arguments();
	foreach( QString tmp, args )
		qDebug() << "Argument:" << tmp;

	// This is only for the broken way of Qt honouring styles in non-KDE-environment...
	app.setStyle( new QPlastiqueStyle );

	// Plugins Library
	//
	// This should probably get into some generic lib-loader that also reads
	// text-files for configuration...
	foreach( QString libname, QStringList() << "libplugins" << "libdopplerplugins" ) {
		QLibrary* lib = new QLibrary( libname, &app );
		lib->load();
		if ( lib->isLoaded() ) {
			qDebug() << QString( "Found lib '%1'!" ).arg( libname );
			typedef void ( *MyPrototype )();
			MyPrototype func = ( MyPrototype ) lib->resolve( "start" );
			if ( func )
				func();
		}
		else
			qWarning() << endl << QString( "Unable to load plugins from lib '%1'!\n error was\"%2\"\n" ).arg( libname ).arg( lib->errorString() );
	}

	//
	// Start
	EodeMainWindow* mw = new EodeMainWindow();

	mw->show();

	if ( args.size() > 1 )
		mw->openXMLFile( args.back() );

	int ret = app.exec();

	delete mw;
	qDebug() << "EvaluationChain finished.";
	return ret;
};

