/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "abstracttreeitem.h"

namespace NSCore {

	AbstractTreeItem::AbstractTreeItem( QString name, AbstractTreeItem* parent )
		: _parent( parent )
		, _name( name )
	{
		//qDebug() << "AbstractTreeItem::AbstractTreeItem(" << name << "," << parent << ")";
		if ( _parent )
			_parent->_childs.append( this );
	}
	AbstractTreeItem::AbstractTreeItem( AbstractTreeItem& item, AbstractTreeItem* parent )
		: _parent( parent )
		, _name( item._name )
	{
		qDebug() << "AbstractTreeItem::AbstractTreeItem(" << &item << "," << parent << ")\n ! Copy constructor is not working right!";
		if ( _parent )
			_parent->_childs.append( this );
	}
	AbstractTreeItem::~AbstractTreeItem() {
		//qDebug() << "AbstractTreeItem::~AbstractTreeItem() _name=" << _name;
		while ( ! _childs.isEmpty() )
			delete _childs.takeLast();
		//qDebug() << "1";
		if ( _parent )
			_parent->_childs.removeAll( this );
		//qDebug() << "AbstractTreeItem::~AbstractTreeItem() finished";
	}
	QString AbstractTreeItem::path() const {
		QString ret="";
		AbstractTreeItem* item = dynamic_cast<AbstractTreeItem*>( _parent );
		if ( item )
			ret = item->path() + "/";
		return ret + name();
	}
	AbstractTreeItem* AbstractTreeItem::getNamedChild( QString n ) const {
		//qDebug() << "AbstractTreeItem::getNamedChild(" << n << ")";
		//qDebug() << " own name=" << _name << " childlist=" << childlist().join( "," );
		if ( n.startsWith( _name ) )
			n = n.section( "/", 1, -1 );
		AbstractTreeItem* item = child( n.section( "/", 0, 0 ) );
		if ( item != 0 ) {
			QString path = n.section( "/", 1, -1 );
			if ( path == "" )
				return item;
			else
				return item->getNamedChild( path );
		}
		//qDebug() << "AbstractTreeItem::getNamedChild()  returning 0!";
		return 0;
	}
	AbstractTreeItem* AbstractTreeItem::child( QString n ) const {
		foreach( AbstractTreeItem* item, _childs )
			if ( item->_name == n )
				return item;
		return 0;
	}

};

