/*
    Copyright 2007-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>
                        Nicki Hinsche <nicvok@freenet.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DOPPLERPLUGINS_H
#define DOPPLERPLUGINS_H

#include "plugins_interface.h"

//#include "interpolation_helpers.h"

class TimeConstantAnalyzer : public NSEvaluation::Analyzer
{
	public:
		TimeConstantAnalyzer(  NSCore::QTreeItem* );

		void analyze(  NSCore::AcqisitionDataSet& data );

		QString type() const { return "TimeConstantAnalyzer"; }

	private:
		double lin_regression(  int x_max, NSCore::ChannelDataSet*, int n_0 ) const;
};

class IntegrationAnalyzer : public NSEvaluation::Analyzer
{
	public:
		IntegrationAnalyzer( NSCore::QTreeItem* );

		void analyze( NSCore::AcqisitionDataSet& data );

		QString type() const { return "IntegrationAnalyzer"; }

	private:
		double sumup( int x_max, int x_min, NSCore::ChannelDataSet*, int n_0 ) const;
		double check_cf( double lfp, double hfp, int x_max, int x_min, NSCore::ChannelDataSet*, int n_0 ) const;
		double check_cfV2( double lfp, double hfp, int x_min, int x_max, double mean, double deviation, NSCore::ChannelDataSet* ch, int n_0 ) const;
};


class DopplerAnalyzers_Client : public NSEvaluation::PluginFactory_Client {
	public:
		DopplerAnalyzers_Client() { }
		virtual ~DopplerAnalyzers_Client() { }

		NSEvaluation::Plugin* create( QString s, NSCore::QTreeItem* p ) {
			if ( s == "TimeConstantAnalyzer" )
				return new TimeConstantAnalyzer( p );
			if ( s == "IntegrationAnalyzer" )
				return new IntegrationAnalyzer( p );
			return 0;
		}

		QStringList canCreate() {
			return QStringList()
								 << "TimeConstantAnalyzer"
								 << "IntegrationAnalyzer"
								 ;
		}

		QString type( QString plugin ) {
			if ( canCreate().contains( plugin ) ) {
				return "Analyzer";
			}
			return "";
		}
};

#endif // DOPPLERPLUGINS_H

