/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "parameter_gui.h"
#include "parameter_gui.moc"

#include <namedparametertreeitem.h>
#include <QtCore/QDebug>
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QLabel>
#include <QtGui/QSpinBox>
#include <QtGui/QDoubleSpinBox>
#include <QtGui/QCheckBox>
#include <QtGui/QLineEdit>
#include <QtGui/QPushButton>

QWidget* ParameterGui::initself( QString st, NSCore::NamedParameterTreeItem* param, QWidget* p )
{
	QWidget* w = new QWidget( p );
	//setAutoFillBackground( true );
	//qDebug() << "ParameterGui::ParameterGui(" << st << "," << long( param ) << "," << long( p ) << ")";
	QHBoxLayout* _layout = new QHBoxLayout( w );
	_layout->setMargin( 2 );
	_layout->setSpacing( 2 );

	if ( _param->type() == QVariant::Int )
	{
		_layout->addWidget( new QLabel( QString( "%1: %2 [%3,%4]" ).arg( st ).arg( param->toString() ).arg( param->min().toString() ).arg( param->max().toString() ), w ), 1 );
		QSpinBox* _spinbox = new QSpinBox( w );
		_layout->addWidget( _spinbox, 1 );
		if ( _param->min().type() != QVariant::Invalid )
			_spinbox->setMinimum( _param->min().toInt() );
		if ( _param->max().type() != QVariant::Invalid )
			_spinbox->setMaximum( _param->max().toInt() );
		_spinbox->setValue( _param->toInt() );
		connect( _spinbox, SIGNAL( valueChanged( int ) ), _param, SLOT( setValue( int ) ) );
		connect( _param, SIGNAL( valueChanged( int ) ), _spinbox, SLOT( setValue( int ) ) );
		if ( _param->guiHint() & NSCore::NamedParameterTreeItem::ProvideReset ) {
			QPushButton* btn = new QPushButton( "Reset", this );
			_layout->addWidget( btn, 0 );
			connect( btn, SIGNAL( clicked() ), this, SLOT( resetCounter() ) );
		}
	}
	else if ( _param->type() == QVariant::Double )
	{
		_layout->addWidget( new QLabel( QString( "%1: %2 [%3,%4]" ).arg( st ).arg( param->toString() ).arg( param->min().toString() ).arg( param->max().toString() ), w ) );
		QDoubleSpinBox* _spin = new QDoubleSpinBox( w );
		_spin->setDecimals( 3 );
		_layout->addWidget( _spin );
		_spin->setSingleStep( 0.1 );
		if ( _param->min().type() != QVariant::Invalid )
			_spin->setMinimum( _param->min().toDouble() );
		if ( _param->max().type() != QVariant::Invalid )
			_spin->setMaximum( _param->max().toDouble() );
		_spin->setValue( _param->toDouble() );
		connect( _spin, SIGNAL( valueChanged( double ) ), _param, SLOT( setValue( double ) ) );
		connect( _param, SIGNAL( valueChanged( double ) ), _spin, SLOT( setValue( double ) ) );
	}
	else if ( _param->type() == QVariant::Bool )
	{
		setAutoFillBackground( true );
		QCheckBox* _check = new QCheckBox( st, w );
		_layout->addWidget( _check );
		_check->setTristate( false );
		_check->setChecked( _param->toBool() );
		connect( _check, SIGNAL( clicked( bool ) ), _param, SLOT( setValue( bool ) ) );
		connect( _param, SIGNAL( valueChanged( bool ) ), _check, SLOT( setChecked( bool ) ) );
	}
	else if ( _param->type() == QVariant::Invalid )
	{
		if ( st != "" )
			_layout->addWidget( new QLabel( "<b>" + st + "</b>", w ) );
	}
	else if ( _param->type() == QVariant::String )
	{
		_layout->addWidget( new QLabel( st, this ) );
		QLineEdit* line = new QLineEdit( _param->toString(), w );
		_layout->addWidget( line );
		connect( line, SIGNAL( textEdited( const QString& ) ), _param, SLOT( setValue( QString ) ) );
		connect( _param, SIGNAL( valueChanged( QString ) ), line, SLOT( setText( const QString& ) ) );
	}
	else
	{
		_layout->addWidget( new QLabel( st, this ) );
		_layout->addWidget( new QLabel( QVariant::typeToName( _param->type() ), this ) );
	}

	if ( !( _param->guiHint() & NSCore::NamedParameterTreeItem::WriteAllowed ) ) {
		foreach( QWidget* w, findChildren<QWidget*>() ) {
			if ( ! dynamic_cast<QPushButton*>( w ) && ! dynamic_cast<QLabel*>( w ) )
				w->setEnabled( false );
		}
	}
	return w;
}
ParameterGui::ParameterGui( NSCore::NamedParameterTreeItem* param, QWidget* p ) : QFrame( p ), _param( param ) {
	//setAutoFillBackground( true );
	//qDebug() << "ParameterGui::ParameterGui(" << long( param ) << "," << long( p ) << ")";
	QVBoxLayout* _layout = new QVBoxLayout( this );
	_layout->setMargin( 1 );
	_layout->setSpacing( 2 );

	setToolTip( param->toolTip() );

	if ( !( param->childs() == 0 && param->type() == QVariant::Invalid ) ) {
		QWidget* own = initself( param->objectName(), param, this );
		_layout->addWidget( own, 1 );
	}

	if ( param->childs() > 0 ) {
		setFrameStyle( QFrame::Plain|QFrame::Panel );
		setLineWidth( 1 );

		foreach( QString name, param->childlist() )
			_layout->addWidget( new ParameterGui( param->getParameter( name ), this ), 1 );
	}
}
ParameterGui::~ParameterGui() {
	//qDebug() << "ParameterGui::~ParameterGui()";
}

void ParameterGui::resetCounter() {
	_param->setValue( 0 );
}

