/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MODIFIERS_H
#define MODIFIERS_H

#include "plugins_interface.h"

class AbsModifier : public NSEvaluation::Modifier
{
	public:
		AbsModifier( NSCore::QTreeItem* );

		void modify( NSCore::AcqisitionDataSet& );

		QString type() const { return "AbsModifier"; }
};

class NormalizeModifier : public NSEvaluation::Modifier
{
	public:
		NormalizeModifier( NSCore::QTreeItem* );

		void modify( NSCore::AcqisitionDataSet& );

		QString type() const { return "NormalizeModifier"; }
};

class FloatingAverageModifier : public NSEvaluation::Modifier
{
	public:
		FloatingAverageModifier( NSCore::QTreeItem* );

		void modify( NSCore::AcqisitionDataSet& );

		QString type() const { return "FloatingAverageModifier"; }
};

class IntegrateModifier : public NSEvaluation::Modifier
{
	public:
		IntegrateModifier( NSCore::QTreeItem* );

		void modify( NSCore::AcqisitionDataSet& );

		QString type() const { return "IntegrateModifier"; }
};

class InterpolatedMinimumAnalyzer;

class PulseIntegrate : public NSEvaluation::Modifier
{
	public:
		PulseIntegrate( NSCore::QTreeItem* );
		void modify( NSCore::AcqisitionDataSet& );
		QString type() const { return "PulseIntegrate"; }
	private:
		InterpolatedMinimumAnalyzer* _analyzer;
};

class SignedIntegrate : public NSEvaluation::Modifier
{
	public:
		SignedIntegrate( NSCore::QTreeItem* );
		void modify( NSCore::AcqisitionDataSet& );
		QString type() const { return "SignedIntegrate"; }
};

class CrossCorrelation : public NSEvaluation::Modifier
{
	public:
		CrossCorrelation( NSCore::QTreeItem* );
		void modify( NSCore::AcqisitionDataSet& );
		QString type() const { return "CrossCorrelation"; }
};

class Modifiers_Client : public NSEvaluation::PluginFactory_Client {
	public:
		NSEvaluation::Plugin* create( QString s, NSCore::QTreeItem* p ) {
			if ( s == "AbsModifier" )
				return new AbsModifier( p );
			if ( s == "NormalizeModifier" )
				return new NormalizeModifier( p );
			if ( s == "FloatingAverageModifier" )
				return new FloatingAverageModifier( p );
			if ( s == "IntegrateModifier" )
				return new IntegrateModifier( p );
			if ( s == "PulseIntegrate" )
				return new PulseIntegrate( p );
			if ( s == "SignedIntegrate" )
				return new SignedIntegrate( p );
			if ( s == "CrossCorrelation" )
				return new CrossCorrelation( p );
			return 0;
		}
		QStringList canCreate() {
			return QStringList()
				<< "AbsModifier"
				<< "NormalizeModifier"
				<< "FloatingAverageModifier"
				<< "IntegrateModifier"
				<< "PulseIntegrate"
				<< "SignedIntegrate"
				<< "CrossCorrelation"
				;
		}

		QString type( QString plugin ) {
			if ( canCreate().contains( plugin ) ) {
				return "Modifier";
			}
			return "";
		}
};

#endif // MODIFIERS_H

