/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#include "spectrumgui.h"
#include "spectrumgui.moc"

#include "spectrumgui_base.h"

#include <QtGui/QTableView>
#include "spectrummodel.h"
#include <chartview.h>
#include <xyspecview.h>
#include <parameter_gui.h>
#include <QtGui/QFileDialog>
#include <QtCore/QDebug>
#include <QtGui/QMessageBox>
#include <QtGui/QLabel>
#include <QtGui/QSpinBox>
#include <QtGui/QLineEdit>

SpectrumGui::SpectrumGui( QWidget* p )
	: QFrame( p )
	, _model( 0 )
	, _table( new QTableView( this ) )
	, _chart( new ChartView( this ) )
	, _parameters( 0 )
{
	_base = new Ui::SpectrumGUI_Base();
	_base->setupUi( this );
	while ( _base->tabs->count() > 0 ) {
		delete _base->tabs->widget( 0 );
		_base->tabs->removeTab( 0 );
	}
	_base->tabs->addTab( _chart, "Graph" );
	_base->tabs->addTab( _table, "Table" );
	connect( _base->btnSave, SIGNAL( clicked() ), this, SLOT( saveToFile() ) );
}
SpectrumGui::~SpectrumGui() {
}

void SpectrumGui::setModel( AbstractSpectrumModel* model ) {
	_model = model;
	if ( model != 0 ) {
		connect( _base->btnClear, SIGNAL( clicked() ), model, SLOT( clearSpectrum() ) );
		if ( _model->dimensions() == 2 ) {
			delete _chart;
			_chart = new XYSpecView( this );
			_base->tabs->insertTab( 0, _chart, "Graph" );
		}
		if ( _parameters != 0 )
			delete _parameters;
		_parameters = new ParameterGui( *( _model->config() ), this );
		_base->tabs->addTab( _parameters, "Parameters" );
	} else {
		delete _parameters;
		_parameters = 0;
	}
	_table->setModel( _model );
	_chart->setModel( _model );
}

void SpectrumGui::saveToFile() {
	if ( _model ) {
		QString filename;
		filename = QFileDialog::getSaveFileName( this, "Save Spectrum to file", "", _model->supportedFilters() + "\nAll files (*)" );
		if ( !filename.isNull() )
			_model->saveToFile( filename );
	}
}

