/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; version 3 of the License.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; see the files lgpl-3.0.txt and
    gpl-3.0.txt. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SPECTRUMGUI_H
#define SPECTRUMGUI_H

#include <QtGui/QFrame>

class QTableView;
class QAbstractItemView;
class AbstractSpectrumModel;
class ParameterGui;

namespace Ui {
	class SpectrumGUI_Base;
};

/**
 * @brief A graphical frontend for the SpectrumModel
 *
 * This class provides a frontend for the SpectrumModel, it allow to view the
 * data as table or chart, save the spectrum and reset it.
 */
class SpectrumGui : public QFrame
{
	Q_OBJECT
	public:
		/// constructor
		SpectrumGui( QWidget* );
		/// destructor
		~SpectrumGui();

		/**
		 * @brief Set the Model that is to be controlled
		 *
		 * @param model The SpectrumModel this gui should represent from now on
		 */
		void setModel( AbstractSpectrumModel* model );

	private slots:
		void saveToFile();

	private:
		AbstractSpectrumModel* _model;
		QTableView* _table;
		QAbstractItemView* _chart;
		ParameterGui* _parameters;

		Ui::SpectrumGUI_Base* _base;
};

#endif // SPECTRUMGUI_H

