/*
    Copyright 2005-2008 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.

*/

#include <QtCore/QDebug>
#include <QtCore/QSettings>
#include <QtGui/QApplication>
#include <QtGui/QWidget>
#include <QtGui/QHBoxLayout>
#include <QtGui/QVBoxLayout>
#include <QtGui/QSplitter>
#include <QtGui/QFrame>
#include <QtGui/QRadioButton>

#include <acqiris.h>
#include <acqiristabs_v2_widget.h>
#include <osciwidget_v3.h>
#include <diskiobuffers_bin.h>
#include <diskio_gui.h>
#include <thread_template.h>

#include "simplediskwriter.h"
#include "simplediskwriter.moc"

#include "libplugins/filters.h"

Filters_Client filtersclient;

SimpleDiskWriterGui::SimpleDiskWriterGui( NSAcqiris::Acqiris* acq, QWidget* p )
	: QWidget( p )
	, _acqiris( acq )
	, _writer( new NSDiskIO::EPOSBinWriter( this ) )
	, _settings( new QSettings( "eposlab", "simplediskwriter", this ) )
{
	setupUi( this );
	_writergui = new NSDiskIO::DiskWriterGui( _writer, splitter );

	while ( digitizerTabs->count() > 0 ) {
		delete digitizerTabs->widget( 0 );
		digitizerTabs->removeTab( 0 );
	}

	for ( int i=0; i<acq->digitizers() && i<1; ++i ) {
		digitizerTabs->addTab( new AcqirisTabsV2Widget( acq->digitizer( i ), digitizerTabs ),
		                       QString( "Digitizer %1" ).arg( i ) );
		connect( _writergui, SIGNAL( numberOfDataSetsReached() ), acq->digitizer( i ), SLOT( stopAcqisition() ) );

		QString tmp = _settings->value( QString( "digitizer %1" ).arg( i ) ).toString();
		//qDebug() << QString( "Digitizer %1: %2" ).arg( i ).arg( tmp );
		QDomDocument doc( "digitizer" );
		doc.setContent( tmp );
		NSCore::NamedParameterTreeItem::setValueFromNode( *( _acqiris->digitizer( i )->config() ), doc.documentElement() );
	}

	splitter->setStretchFactor( splitter->count()-1, 1 );

	_osci = new OsciWidgetV3( splitter );
	splitter->setStretchFactor( splitter->count()-1, 255 );

	connect( radioDisk, SIGNAL( toggled( bool ) ), this, SLOT( toggleDisk( bool ) ) );
	connect( radioOsci, SIGNAL( toggled( bool ) ), this, SLOT( toggleOsci( bool ) ) );
	radioOsci->setChecked( true );
}
SimpleDiskWriterGui::~SimpleDiskWriterGui() {
	qDebug() << "SimpleDiskWriterGui::~SimpleDiskWriterGui()";
	for ( int i=0; i<_acqiris->digitizers(); ++i ) {
		//qDebug() << QString( "Digitizer %1: %2" ).arg( i ).arg(  _acqiris->digitizer( i )->config()->toXML() );
		_settings->setValue( QString( "digitizer %1" ).arg( i ), _acqiris->digitizer( i )->config()->toXML() );
	}
}


void SimpleDiskWriterGui::toggleDisk( bool n ) {
	qDebug() << "SimpleDiskWriterGui::toggleDisk( bool" << n << ") [defunct]";
	if ( n ) {
		//_writergui->setNewFilename();
		_acqiris->digitizer( 0 )->registerConsumer( _writer );
	} else
		_writer->setFile( "" );
}
void SimpleDiskWriterGui::toggleOsci( bool n ) {
	qDebug() << "SimpleDiskWriterGui::toggleOsci( bool" << n << ")";
	if ( n ) {
		_acqiris->digitizer( 0 )->registerConsumer( _osci->chainMember() );
	}
}

int main( int argc, char** argv )
{
	QApplication* app = new QApplication( argc, argv );

	NSCore::Thread_Template<NSAcqiris::Acqiris>* acqiris = new NSCore::Thread_Template<NSAcqiris::Acqiris>( app );
	acqiris->start();

	while ( acqiris->item() == 0 )
		usleep( 50 );

	SimpleDiskWriterGui* gui = new SimpleDiskWriterGui( acqiris->item(), 0 );

	gui->show();
	int ret = app->exec();

	delete gui;

	return ret;
}

